package de.ciluvien.mensen.ui.components

import androidx.compose.foundation.BorderStroke
import androidx.compose.foundation.background
import androidx.compose.foundation.isSystemInDarkTheme
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.IntrinsicSize
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.Card
import androidx.compose.material3.CardDefaults
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBarDefaults
import androidx.compose.material3.TopAppBarScrollBehavior
import androidx.compose.material3.pulltorefresh.PullToRefreshContainer
import androidx.compose.material3.pulltorefresh.PullToRefreshState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.draw.rotate
import androidx.compose.ui.draw.scale
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.input.nestedscroll.nestedScroll
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.unit.dp
import androidx.navigation.NavController
import coil.compose.AsyncImage
import de.ciluvien.mensen.R
import de.ciluvien.mensen.data.local.Meal
import de.ciluvien.mensen.ui.Screen

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun MenuList(
    navController: NavController,
    pullToRefreshState: PullToRefreshState,
    scrollBehavior: TopAppBarScrollBehavior = TopAppBarDefaults.pinnedScrollBehavior(),
    menu: List<Meal>,
    canteenName: String
) {
    Box(modifier = Modifier
        .nestedScroll(pullToRefreshState.nestedScrollConnection)
        .nestedScroll(scrollBehavior.nestedScrollConnection)
    ){
        LazyColumn(
            modifier = Modifier
                .fillMaxSize()
                .padding(horizontal = 5.dp),
            verticalArrangement = Arrangement.spacedBy(10.dp)
        ) {
            if(!pullToRefreshState.isRefreshing){
                items(
                    items = menu,
                    key = {meal ->
                        meal.id
                    }
                ) {meal ->
                    MenuListItem(navController, meal = meal, canteenName)
                }
                item{Spacer(modifier = Modifier.size(80.dp))}
            }
        }
        PullToRefreshContainer(
            state = pullToRefreshState,
            modifier = Modifier.align(Alignment.TopCenter)
        )
    }
}

@Composable
fun MenuListItem(navController: NavController, meal: Meal, canteenName: String) {
    val context = LocalContext.current

    Card(
        colors = CardDefaults.cardColors(
            containerColor = MaterialTheme.colorScheme.surfaceContainer
        ),
        border = BorderStroke(2.dp, MaterialTheme.colorScheme.surfaceContainer),
        onClick = { navController.navigate("${Screen.MealDetailsScreen.route}/${canteenName}/${meal.id}") }
    ) {
        Row(
            verticalAlignment = Alignment.CenterVertically,
            horizontalArrangement = Arrangement.SpaceEvenly,
            modifier = Modifier
                .padding(2.dp)
                .padding(end = 5.dp)
                .height(IntrinsicSize.Min)
        ) {
            val isGenericImage = meal.imageULR.contains("lieber-mensen")
            if (isGenericImage) {
                Column(
                    verticalArrangement = Arrangement.Center,
                    modifier = Modifier
                        .fillMaxHeight()
                        .clip(RoundedCornerShape(10.dp))
                        .background(Color.White)
                ) {
                    AsyncImage(
                        model = meal.imageULR,
                        contentDescription = context.resources.getText(R.string.meal_image).toString(),
                        contentScale = ContentScale.Crop,
                        modifier = Modifier
                            .size(120.dp, 80.dp)
                    )
                }
            } else {
                AsyncImage(
                    model = meal.imageULR,
                    contentDescription = context.resources.getText(R.string.meal_image).toString(),
                    contentScale = ContentScale.Crop,
                    modifier = Modifier
                        .size(120.dp, 80.dp)
                        .clip(RoundedCornerShape(10.dp))
                )

            }

            Spacer(modifier = Modifier.size(5.dp))

            Column(
                modifier = Modifier
                    .padding(5.dp)
                    .fillMaxHeight(),
                verticalArrangement = Arrangement.SpaceBetween
            ){
                Text(
                    text = meal.name,
                    style = MaterialTheme.typography.bodyLarge
                )

                Spacer(Modifier.size(5.dp))

                Row(
                    modifier = Modifier.fillMaxWidth(),
                    horizontalArrangement = Arrangement.SpaceBetween,
                    verticalAlignment = Alignment.CenterVertically
                ) {
                    if(meal.isSoldOut == true) {
                        Text(
                            text = context.resources.getText(R.string.meal_sold_out).toString(),
                            style = MaterialTheme.typography.labelLarge,
                            color = MaterialTheme.colorScheme.onErrorContainer
                        )
                    } else {
                        Text(
                            text = remember {
                                meal.prices.values.sorted().joinToString(separator = "€/ ", postfix = "€")
                            },
                            style = MaterialTheme.typography.labelLarge,
                            color = MaterialTheme.colorScheme.onSurfaceVariant
                        )
                    }

                    Row {
                        if(meal.isVegan) {
                            Icon(
                                painter = painterResource(id = R.drawable.leaf_24),
                                contentDescription = context.resources.getText(R.string.vegan).toString(),
                                modifier = Modifier.scale(0.75f)
                            )
                        } else if(meal.isVegetarian) {
                            Icon(
                                painter = painterResource(id = R.drawable.carrot_24),
                                contentDescription = context.resources.getText(R.string.vegetarian).toString(),
                                modifier = Modifier.scale(0.75f)
                            )
                        }

                        val isEveningOffer = remember {
                            meal.isEveningOffer()
                        }
                        if(isEveningOffer) {
                            Icon(
                                painter = painterResource(id = R.drawable.baseline_access_time_24),
                                contentDescription = context.resources.getText(R.string.menu_evening_offer).toString(),
                                modifier = Modifier
                                    .rotate(-120f)
                                    .scale(0.75f)
                            )
                        }
                    }
                }
            }
        }
    }
}
