package de.ciluvien.mensen.ui.screens

import android.nfc.NfcAdapter
import android.nfc.TagLostException
import android.nfc.tech.IsoDep
import android.util.Log
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.outlined.ArrowBack
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.navigation.NavController
import de.ciluvien.mensen.R
import de.ciluvien.mensen.data.nfc.NfcBroadcastReceiver
import de.ciluvien.mensen.ui.components.core.CustomTextIcon
import de.ciluvien.mensen.ui.components.core.CustomTopAppBar
import java.io.IOException
import java.math.BigDecimal


val SELECT_APP: ByteArray = byteArrayOf(
    0x90.toByte(), // CLA
    0x5A.toByte(), // SELECT APP
    0x00.toByte(), // P1
    0x00.toByte(), // P2
    0x03.toByte(), // Data Length
    0x5F.toByte(), 0x84.toByte(), 0x15.toByte(), // AppId
    0x00.toByte(), // Maximum data answer length
)

val READ_VALUE: ByteArray = byteArrayOf(
    0x90.toByte(), // CLA
    0x6C.toByte(), // READ VALUE
    0x00.toByte(), // P1
    0x00.toByte(), // P2
    0x01.toByte(), // Data Length
    0x01.toByte(), // File
    0x00.toByte(), // Maximum data answer length
)

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun CardScreen(navController: NavController) {
    val context = LocalContext.current

    Scaffold(
        topBar = {
            CustomTopAppBar(
                title = context.resources.getText(R.string.card_title).toString(),
                leftIcon = CustomTextIcon(
                    onClick = { navController.popBackStack() },
                    icon = Icons.AutoMirrored.Outlined.ArrowBack,
                    description = context.resources.getText(R.string.canteen_details_back).toString()
                ),
            )
        }
    ) {padding ->
        Box(modifier = Modifier.padding(padding)) {
            val balance = remember {
                mutableStateOf("")
            }
            NfcBroadcastReceiver { genericTag ->
                val tag = IsoDep.get(genericTag)
                try {
                    tag.connect()
                    if (tag.isConnected) {
                        Log.i("nfc", "Selecting app")
                        val appResult = tag.transceive(SELECT_APP)
                        if (appResult[0] != 0x91.toByte()) {
                            Log.w("nfc", "Error selecting app: $appResult")
                            return@NfcBroadcastReceiver
                        }

                        Log.i("nfc", "Reading value")
                        val valueResult = tag.transceive(READ_VALUE)
                        if (valueResult.size >= 2 && valueResult[valueResult.size - 2] != 0x91.toByte()) {
                            Log.w("nfc", "Error reading value: $valueResult")
                            return@NfcBroadcastReceiver
                        }
                        tag.close()

                        balance.value = valueResult
                            .dropLast(2)
                            .reversed()
                            .toByteArray()
                            .toSingleInt()
                            .toBigDecimal()
                            .divide(BigDecimal.valueOf(1000))
                            .toString()
                    }
                } catch (e: TagLostException) {
                    Log.w("nfc", "Error: Tag lost connection")
                } catch (e: IOException) {
                    Log.w("nfc", "Error: ${e.message}")
                } catch (e: SecurityException) {
                    Log.w("nfc", "Error: ${e.message}")
                } catch (e: Exception) {
                    Log.w("nfc", "Error: ${e.message}")
                }
            }

            val nfcAdapter = NfcAdapter.getDefaultAdapter(context.applicationContext)
            if (nfcAdapter == null) {
                CenteredTexts(
                    texts = listOf(
                        context.resources.getText(R.string.card_nfc_not_supported).toString()
                    )
                )
            } else if (!nfcAdapter.isEnabled) {
                CenteredTexts(
                    texts = listOf(
                        context.resources.getText(R.string.card_nfc_disabled).toString(),
                    )
                )
            } else if (balance.value == "") {
                CenteredTexts(
                    texts = listOf(
                        context.resources.getText(R.string.card_hold_near_phone).toString(),
                    )
                )
            } else {
                CenteredTexts(
                    texts = listOf(
                        "${context.resources.getText(R.string.card_current_balance)}:",
                        "${balance.value} €"
                    )
                )
            }
        }
    }
}

@Composable
fun CenteredTexts(texts: List<String>) {
    Column(
        verticalArrangement = Arrangement.Center,
        horizontalAlignment = Alignment.CenterHorizontally,
        modifier = Modifier
            .fillMaxSize()
            .padding(20.dp)
    ) {
        texts.forEach {text ->
            Text(
                text = text,
                color = MaterialTheme.colorScheme.onSurface,
                style = MaterialTheme.typography.headlineSmall,
                textAlign = TextAlign.Center
            )
            Spacer(modifier = Modifier.size(20.dp))
        }
    }
}

fun ByteArray.toSingleInt(): Int {
    return this.fold(0) { acc, byte ->
        (acc shl 8) or (byte.toInt() and 0xFF)
    }
}
